# Standard Library
import logging

import premailer

# Websauna
from websauna.system.core.utils import get_secrets
from websauna.system.model.retry import retryable
from websauna.system.task.tasks import ScheduleOnCommitTask
from websauna.system.task.tasks import task
from websauna.utils.time import now

from .importer import import_all_users
from .interfaces import INewsletterGenerator
from .mailgun import Mailgun
from .state import NewsletterState


logger = logging.getLogger(__name__)


@task(base=ScheduleOnCommitTask, bind=True)
def send_newsletter_task(self: ScheduleOnCommitTask, subject, preview_email, testmode, now_, import_subscribers):
    """Do user import and newsletter inside a Celery worker process.

    We carefully split transaction handling to several parts.
    """

    # from celery.contrib import rdb ; rdb.set_trace()

    request = self.get_request()

    secrets = get_secrets(request.registry)

    if not now_:
        now_ = now()

    mailing_list = secrets["mailgun.mailing_list"]

    if preview_email:
        to = preview_email
        subject = "[PREVIEW] " + subject
    else:
        to = mailing_list

    newsletter = request.registry.queryAdapter(request, INewsletterGenerator)

    state = NewsletterState(request)

    text = "Please see the attached HTML mail."

    @retryable(tm=request.tm)
    def render_tx():
        """Run HTML rendering in its own transaction, as it most likely reads database."""
        return newsletter.render(since=state.get_last_send_timestamp())

    html = render_tx()
    html = premailer.transform(html)

    from_ = secrets["mailgun.from"]
    domain = secrets["mailgun.domain"]
    campaign = now().isoformat()

    mailgun = Mailgun(request.registry)

    if import_subscribers:
        # This may take a looooong time....
        logger.info("Importing subscribers")
        import_all_users(mailgun, request.dbsession, mailing_list, tm=request.tm)

    logger.info("Sending out newsletter %s %s %s %s %s", domain, subject, to, from_, campaign)
    mailgun.send(domain, to, from_, subject, text, html, campaign)

    if not preview_email:
        # Only mark newsletter send if not preview
        state.set_last_send_timestamp(now_)



# Pyramid
from pyramid.config import Configurator

# Websauna
from websauna.system import Initializer
from websauna.utils.autoevent import after
from websauna.utils.autoevent import bind_events


class AddonInitializer:
    """Configure this addon for websauna.

    * We hook to existing parts of initialization process using ``@after`` aspect advisor

    * For something we don't have a direct join point we just initialize through ``run()``
    """

    def __init__(self, config: Configurator):
        self.config = config

    @after(Initializer.configure_templates)
    def configure_templates(self):
        """Include our package templates folder in Jinja 2 configuration."""
        # Use prepend=False here so that the app can override our templates easily
        self.config.add_jinja2_search_path('websauna.newsletter:templates', name='.html', prepend=False)  # HTML templates for pages

    @after(Initializer.configure_tasks)
    def configure_tasks(self):
        from . import tasks
        self.config.scan(tasks)

    @after(Initializer.configure_admin)
    def configure_admin(self):
        from . import adminviews
        from . import menu
        self.config.scan(adminviews)
        self.config.scan(menu)

    def configure_addon_views(self):
        from . import views
        self.config.scan(views)

    def run(self):

        # This will make sure our initialization hooks are called later
        bind_events(self.config.registry.initializer, self)

        # Run our custom initialization code which does not have a good hook
        self.configure_addon_views()


def includeme(config: Configurator):
    """Entry point for Websauna main app to include this addon.

    In the Initializer of your app you should have:

        def include_addons(self):
            # ...
            self.config.include("websauna.newsletter")

    """
    addon_init = AddonInitializer(config)
    addon_init.run()



"""Newsletter admin inteface."""

# Pyramid
import colander
import deform
from pyramid import httpexceptions
from pyramid.response import Response
from pyramid.view import view_config

# Websauna
from websauna.newsletter.interfaces import INewsletterGenerator
from websauna.system.admin.admin import Admin
from websauna.system.core import messages
from websauna.system.core.utils import get_secrets
from websauna.system.form.resourceregistry import ResourceRegistry
from websauna.system.form.schema import CSRFSchema
from websauna.system.http import Request
from websauna.system.user.utils import get_user_class

from .mailgun import MailgunError
from .sender import send_newsletter
from .state import NewsletterState


class NewsletterSend(CSRFSchema):
    """Send a news letter."""

    subject = colander.SchemaNode(colander.String(), title="Newsletter subject")

    preview = colander.SchemaNode(colander.Boolean(), description="Is this a preview send.", default=True)

    import_subscribers = colander.SchemaNode(colander.Boolean(), description="Import userbase as new subscribers", default=True)

    email = colander.SchemaNode(colander.String(), title="Preview email", description="Send preview email to this email address", validator=colander.Email(), missing=colander.null)

    domain = colander.SchemaNode(
        colander.String(),
        missing=colander.null,
        widget=deform.widget.TextInputWidget(readonly=True),
        title="Mailgun outbound domain",
        description="From secrets.ini",
    )

    mailing_list = colander.SchemaNode(
        colander.String(),
        missing=colander.null,
        title="Mailgun mailing list email",
        widget=deform.widget.TextInputWidget(readonly=True),
        description="From secrets.ini",
    )

    api_key = colander.SchemaNode(
        colander.String(),
        missing=colander.null,
        widget=deform.widget.TextInputWidget(readonly=True),
        title="Mailgun API key",
        description="From secrets.ini",
    )

    def validator(self, node: "NewsletterSend", appstruct: dict):
        """Custom schema level validation code."""

        # appstruct is Colander appstruct after all other validations have passed
        # Note that this method may not be never reached
        if appstruct["preview"] and appstruct["email"] == colander.null:
            # This error message appears at the top of the form
            raise colander.Invalid(node["email"], "Please fill in email field if you want to send a preview email.")


@view_config(context=Admin,
    name="newsletter",
    route_name="admin",
    permission="edit",
    renderer="newsletter/admin.html")
def newsletter(context: Admin, request: Request):
    """Newsletter admin form."""
    schema = NewsletterSend().bind(request=request)

    # Create a styled button with some extra Bootstrap 3 CSS classes
    b = deform.Button(name='process', title="Send", css_class="btn-block btn-lg btn-primary")
    form = deform.Form(schema, buttons=(b, ), resource_registry=ResourceRegistry(request))

    secrets = get_secrets(request.registry)
    api_key = secrets.get("mailgun.api_key", "")
    domain = secrets.get("mailgun.domain", "")
    mailing_list = secrets.get("mailgun.mailing_list", "")
    preview_url = request.resource_url(context, "newsletter-preview")

    # TODO: Use user registry here
    User = get_user_class(request.registry)
    user_count = request.dbsession.query(User).count()

    state = NewsletterState(request)

    rendered_form = None

    # User submitted this form
    if request.method == "POST":
        if 'process' in request.POST:

            try:
                appstruct = form.validate(request.POST.items())

                if appstruct["preview"]:
                    send_newsletter(request, appstruct["subject"], preview_email=appstruct["email"], import_subscribers=appstruct["import_subscribers"])
                    messages.add(request, "Preview email sent.")
                else:
                    send_newsletter(request, appstruct["subject"], import_subscribers=appstruct["import_subscribers"])
                    messages.add(request, "Newsletter sent.")

                return httpexceptions.HTTPFound(request.url)

            except MailgunError as e:
                # Do a form level error message
                exc = colander.Invalid(form.widget, "Could not sent newsletter:" + str(e))
                form.widget.handle_error(form, exc)

            except deform.ValidationFailure as e:
                # Render a form version where errors are visible next to the fields,
                # and the submitted values are posted back
                rendered_form = e.render()
        else:
            # We don't know which control caused form submission
            return httpexceptions.HTTPBadRequest("Unknown form button pressed")

    # Render initial form
    # Default values for read only fields
    if rendered_form is None:
        rendered_form = form.render({
            "api_key": api_key,
            "domain": domain,
            "mailing_list": mailing_list,
        })

    # This loads widgets specific CSS/JavaScript in HTML code,
    # if form widgets specify any static assets.
    form.resource_registry.pull_in_resources(request, form)

    return locals()


@view_config(context=Admin,
    name="newsletter-preview",
    route_name="admin",
    permission="edit")
def newsletter_preview(request: Request):
    """Render the preview of current outgoing newsletter inside an <iframe>."""

    newsletter = request.registry.queryAdapter(request, INewsletterGenerator)
    if not newsletter:
        return Response("INewsletterGenerator not configured")

    state = NewsletterState(request)

    return Response(newsletter.render(state.get_last_send_timestamp()))



# Pyramid
from pyramid.events import subscriber

# Websauna
from websauna.system.admin.events import AdminConstruction
from websauna.system.admin.menu import TraverseEntry


@subscriber(AdminConstruction)
def contribute_model_admin(event):
    """Create newsletter menu entry in the admin interface."""

    admin = event.admin

    menu = admin.get_admin_menu()
    entry = TraverseEntry("admin-menu-newsletter", label="Newsletter", resource=admin, name="newsletter", icon="fa-envelope")
    menu.add_entry(entry)



# Standard Library
from datetime import datetime
from typing import Tuple

# Pyramid
from zope.interface import Interface


class INewsletterGenerator(Interface):
    """Newsletter interface.

    Allow to configure different newsletters for a website.

    Registered as a adapter for request.
    """

    def render(since: datetime) -> Tuple[str, str]:
        """Render the current newsletter payload.

        :param since: When the last newsletter was sent or None if the first outgoing newsletter on this site.

        :return: tuple (subject, html)
        """



"""User state management."""
# Standard Library
import logging
from typing import Optional

# Pyramid
from transaction import TransactionManager

# Websauna
from websauna.system.model.retry import ensure_transactionless
from websauna.system.model.retry import retryable
from websauna.system.user.models import User

from .mailgun import Mailgun


logger = logging.getLogger(__name__)


def import_subscriber(mailgun: Mailgun, address: str, user: User, upsert=True) -> bool:
    """Add one subscriber to the mailing list.

    :return: True if user was fresh and imported
    """

    # Track import status in user_data JSON, so we don't do double requests if the user has already been subscribed once
    mailing_list_subscribes = user.user_data.get("mailing_list_subscribes", [])

    if address not in mailing_list_subscribes:

        # Some sanity logic to filter out emails that are legit in some services, unlegit in Mailgun
        first_part, second_part = address.split("@")
        if first_part.startswith(".") or first_part.endswith("."):
            logger.info("Bad email address: %s", address)
            return False

        logger.info("Subscribing %s to %s", user.email, address)

        # Don't set subscribed field, so that we don't accidentally update unsubscribed users
        data = {
            "address": user.email,
            "name": user.friendly_name,
            "upsert": upsert and "yes" or "no",
        }

        try:
            mailgun.update_subscription(address, data)
        except Exception as e:
            logger.error("Failed to subscribe email %s: %s", user.email, e)
            return False

        mailing_list_subscribes.append(address)

        user.user_data["mailing_list_subscribes"] = mailing_list_subscribes
        return True
    return False


def import_all_users(mailgun: Mailgun, dbsession, address: str, tm: Optional[TransactionManager]=None) -> int:
    """Update Mail subscribers database from Websauna internal database.

    :return: Imported count
    """

    if tm is None:
        tm = dbsession.transaction_manager

    count = 0

    for obj in dbsession:
        print(obj)

    # Make sure we don't have a transaction in progress as we do batching ourselves
    ensure_transactionless(transaction_manager=tm)

    @retryable(tm=tm)
    def tx1():
        """Get user ids on the first transaction."""
        return [u.id for u in dbsession.query(User.id).all()]

    @retryable(tm=tm)
    def tx_n(id):
        """For each user, import it in a subsequent transaction."""
        u = dbsession.query(User).get(id)
        if import_subscriber(mailgun, address, u):
            return 1
        else:
            return 0

    user_ids = tx1()
    for id in user_ids:
        count += tx_n(id)

    logger.info("Imported %d users", count)

    return count



"""Mailgun API integration tests."""
# Standard Library
import datetime

# Pyramid
import transaction

# Websauna
from websauna.newsletter.sender import send_newsletter
from websauna.newsletter.state import NewsletterState
from websauna.system.core.redis import get_redis


def test_send_preview(mailgun, populated_mailing_list, domain, test_request):
    """Send out a news letter preview."""

    # Make sure we run on empty state
    redis = get_redis(test_request)
    redis.delete(NewsletterState.REDIS_NEWSLETTER_TIMESTAMP)

    now = datetime.datetime(1980, 1, 15)

    with transaction.manager:
        # This is triggered only on commit
        send_newsletter(test_request, "Test subject", testmode=True, now_=now)

    state = NewsletterState(test_request)

    assert state.get_last_send_timestamp().year == 1980



# Pyramid
import colander
import deform
from pyramid.httpexceptions import HTTPBadRequest
from pyramid.httpexceptions import HTTPFound

# Websauna
from websauna.system.core import messages
from websauna.system.core.route import simple_route
from websauna.system.core.sitemap import include_in_sitemap
from websauna.system.core.utils import get_secrets
from websauna.system.form.schema import CSRFSchema
from websauna.system.http import Request

from .mailgun import Mailgun


class NewsletterSubscriptionSchema(CSRFSchema):
    email = colander.Schema(colander.String(), validator=colander.Email())


def subscribe_email(request: Request, email: str):
    """Subscribe an email address to our default mailing list.

    Don't change existing subscription status.
    """
    # Save form data from appstruct
    mailgun = Mailgun(request.registry)
    secrets = get_secrets(request.registry)
    address = secrets["mailgun.mailing_list"]

    return mailgun.update_subscription(address, {
        "address": email,
        "email": email,
        "upsert": "yes"
    })


@simple_route("/subscribe-newsletter", route_name="subscribe_newsletter")
@include_in_sitemap(False)
def subscribe_newsletter(request: Request):

    schema = NewsletterSubscriptionSchema().bind(request=request)
    form = deform.Form(schema)

    # User submitted this form
    if request.method == "POST":
        if 'subscribe' in request.POST:
            try:
                appstruct = form.validate(request.POST.items())
                email = appstruct["email"]

                subscribe_email(request, email)

                # Thank user and take him/her to the next page
                messages.add(request, kind="info", html=True, msg="<strong>{}</strong> has been subscribed to the newsletter.".format(email))
                return HTTPFound(request.route_url("home"))

            except deform.ValidationFailure as e:
                # Render a form version where errors are visible next to the fields,
                # and the submitted values are posted back
                messages.add(request, kind="error", msg="Email was not valid")
                return HTTPFound(request.route_url("home"))
        else:
            # We don't know which control caused form submission
            return HTTPBadRequest("Unknown form button pressed")

    return HTTPBadRequest("POST-only endpoint")



# Standard Library
import datetime

# Websauna
from websauna.system.core.redis import get_redis
from websauna.system.http import Request


class NewsletterState:
    """Newsletter state management.

    State is stord within redis.
    """

    #: Redis key when the last news letter was send
    REDIS_NEWSLETTER_TIMESTAMP = "newsletter_sent_timestamp"

    def __init__(self, request: Request):
        self.request = request
        self.redis = get_redis(self.request)

    def set_last_send_timestamp(self, now: datetime.datetime):
        ts = now.timestamp()
        self.redis.set(self.REDIS_NEWSLETTER_TIMESTAMP, ts)

    def get_last_send_timestamp(self) -> datetime.datetime:
        """Get UNIX timestamp when the last newsletter went out."""
        val = self.redis.get(self.REDIS_NEWSLETTER_TIMESTAMP)
        if not val:
            return None

        dt = datetime.datetime.utcfromtimestamp(float(val)).replace(tzinfo=datetime.timezone.utc)
        return dt



"""Mailgun API integration tests."""
import pytest

# Websauna
from websauna.newsletter.mailgun import MailgunError


def test_api_error(dbsession, registry, mailgun):
    """We get exception on API error."""

    with pytest.raises(MailgunError):
        mailgun.delete_list("foobar@not-exist.com")


def test_import_subscribers(dbsession, registry, mailgun, populated_mailing_list):
    """Import Websauna users to Mailgun list."""
    paginator = mailgun.list_members(populated_mailing_list)
    items = paginator["items"]
    assert len(items) == 1
    assert items[0]["address"] == "example@example.com"


def test_send_news_letter(mailgun, populated_mailing_list, domain):
    """Send out a news letter."""

    text = """Please see HTML mail"""
    html = """<a href="%mailing_list_unsubscribe_url%">Unsubscribe</a>"""
    to = populated_mailing_list
    from_ = "example@" + domain
    subject = "Test message"
    domain = domain
    campaign = "TAGGGGED"
    testmode = True

    resp = mailgun.send(domain, to, from_, subject, text, html, campaign, testmode)
    assert "id" in resp



"""This contains app entry point for running a demo site for this addon or running functional tests for this addon."""

# Standard Library
import datetime

# Pyramid
from pyramid.interfaces import IRequest
from pyramid.renderers import render
from zope.interface import implementer

# Websauna
import websauna.system
from websauna.newsletter.interfaces import INewsletterGenerator
from websauna.system.http import Request


@implementer(INewsletterGenerator)
class DemoNewsletterRenderer:
    """Render sample newsletter content."""

    def __init__(self, request: Request):
        self.request = request

    def render(self, since: datetime.datetime):
        """Render some sample content."""
        from websauna.system.user.models import User
        request = self.request
        users = request.dbsession.query(User).all()
        return render("newsletter/demo_newsletter.html", value={"users": users}, request=self.request)


class Initializer(websauna.system.DemoInitializer):
    """A demo / test app initializer for testing addon websauna.newsletter."""

    def include_addons(self):
        """Include this addon in the configuration."""
        self.config.include("websauna.newsletter")

    def configure_templates(self):
        super(Initializer, self).configure_templates()

        # Your app templates go here
        self.config.add_jinja2_search_path('websauna.magiclogin:demotemplates', name='.html', prepend=True)

    def run(self):
        super(Initializer, self).run()

        # Configure newsletter renderer
        registry = self.config.registry
        registry.registerAdapter(factory=DemoNewsletterRenderer, required=(IRequest,), provided=INewsletterGenerator)


def main(global_config, **settings):
    init = Initializer(global_config)
    init.run()
    return init.make_wsgi_app()



"""Mailgun based newsletter management."""
# Standard Library
import logging

# Pyramid
from pyramid.registry import Registry

import requests

# Websauna
from websauna.system.core.utils import get_secrets


logger = logging.getLogger(__name__)


def yesify(val):
    """Because booleans where not invented 150 years ago."""
    return "yes" if val else "no"


class MailgunError(Exception):
    """Mailgun API did not like a particular request."""


class Mailgun:
    """Simple mailgun mailing list API wrapper for Websauna."""

    def __init__(self, registry: Registry, api_url=None, timeout=10):
        secrets = get_secrets(registry)
        self.api_key = secrets["mailgun.api_key"]
        if api_url:
            self.api_url = api_url
        else:
            self.api_url = "https://api.mailgun.net/v3"

        # Open HTTP 1.1 Keep-Alive session
        self.session = requests.Session()
        self.timeout = timeout

    def make_request(self, func: str, method="POST", **data) -> dict:
        """Call Mailgun API.

        :return: Response parsed JSON dict
        """

        url = self.api_url + "/" + func

        resp = self.session.request(
            method,
            url,
            auth=('api', self.api_key),
            data=data,
            timeout=self.timeout)

        if resp.status_code != 200:
            raise MailgunError("Mailgun returned non-200 code for URL {} {}: {} {}".format(method, url, resp.status_code, resp.text))

        return resp.json()

    def create_list(self, address: str, description: str) -> dict:
        """Create a new mailign list"""
        return self.make_request("lists", address=address, description=description)

    def delete_list(self, address: str) -> dict:
        """Delete a new mailign list"""
        return self.make_request("lists/{}".format(address), method="DELETE")

    def list_members(self, address: str) -> dict:
        """Get mailing list members pagination object"""
        return self.make_request("lists/{}/members/pages".format(address), method="GET")

    def update_subscription(self, address: str, data: dict) -> dict:
        """Update subscription information

        http://mailgun-documentation.readthedocs.io/en/latest/api-mailinglists.html#mailing-lists

        :param address: Mailing list
        :param data: dict like {"address": user.email, "name": user.friendly_name, "upsert": upsert and "yes" or "no",}
        """
        return self.make_request("lists/{}/members".format(address), **data)

    def send(self, domain: str, to: str, from_: str, subject: str, text: str, html: str, campaign=None, testmode=False, tracking=True, tracking_clicks=True, tracking_opens=True):
        """Send a newsletter or single message.

        See

        * https://documentation.mailgun.com/user_manual.html#mailing-lists

        * https://documentation.mailgun.com/api-sending.html#sending
        """

        data = {
            "to": to,
            "from": from_,
            "text": text,
            "html": html,
            "subject": subject,
            "testmode": yesify(testmode),
            "o:tracking": yesify(tracking),
            "o:tracking-clicks": yesify(tracking_clicks),
            "o:tracking-opens": yesify(tracking_opens),
        }

        if campaign:
            data["o:campaign"] = yesify(campaign)

        return self.make_request("{}/messages".format(domain), **data)



"""Subscription tests."""

# Websauna
from websauna.newsletter.views import subscribe_email


def test_subscribe_email(mailgun, mailing_list, test_request):
    """Subscribe an email address without a user."""

    subscribe_email(test_request, "example@example.com")

    paginator = mailgun.list_members(mailing_list)
    items = paginator["items"]
    assert len(items) == 1
    assert items[0]["address"] == "example@example.com"



# Pyramid
import transaction

import json
import requests

import pytest

# Websauna
from websauna.newsletter.importer import import_all_users
from websauna.newsletter.mailgun import Mailgun
from websauna.system.core.utils import get_secrets
from websauna.tests.utils import create_user

MAILGUN_RESPONSES = {
    'https://api.mailgun.net/v3/lists/foobar@not-exist.com': {
        'DELETE': {
            'status_code': 404,
            'body': {
                'message': 'Mailing list does not exist',
                'address': 'unit-testing@mailgun.websauna.org'
            }
        }
    },
    'https://api.mailgun.net/v3/lists/unit-testing@mailgun.websauna.org': {
        'DELETE': {
            'status_code': 200,
            'body': {
                'message': 'Mailing list has been deleted',
                'address': 'unit-testing@mailgun.websauna.org'
            }
        }
    },
    'https://api.mailgun.net/v3/mailgun.websauna.org/messages': {
        'POST': {
            'status_code': 200,
            'body': {
                'message': 'Queued. Thank you.',
                'id': '<20111114174239.25659.5817@samples.mailgun.org>'
            }
        }
    },
    'https://api.mailgun.net/v3/lists/unit-testing@mailgun.websauna.org/members': {
        'POST': {
            'status_code': 200,
            'body': {
                'member': {
                    'name': 'example@example.com',
                    'subscribed': True,
                    'address': 'example@example.com'
                },
                'message': 'Mailing list member has been created'
            }
        }
    },
    'https://api.mailgun.net/v3/lists/unit-testing@mailgun.websauna.org/members/pages': {
        'GET': {
            'status_code': 200,
            'body': {
                'items': [
                    {
                        'name': 'example@example.com',
                        'subscribed': True,
                        'address': 'example@example.com'
                    }
                ],
                'paging': {
                    'first': 'https://url_to_first_page',
                    'last': 'https://url_to_last_page',
                    'next': 'http://url_to_next_page',
                    'previous': 'http://url_to_previous_page'
                }
            }
        }
    },
    'https://api.mailgun.net/v3/lists': {
        'POST': {
            'status_code': 200,
            'body': {
                'message': 'Mailing list has been created',
                'list': {
                    'created_at': 'Tue, 06 Mar 2012 05:44:45 GMT',
                    'address': 'unit-testing@mailgun.websauna.org',
                    'members_count': 0,
                    'description': 'Websauna developers list',
                    'name': ''
                }
            }
        }
    },
}


@pytest.fixture
def mailgun(registry):
    m = Mailgun(registry)
    return m


@pytest.fixture()
def mailing_list(mailgun, registry):
    secrets = get_secrets(registry)
    list_address = secrets["mailgun.mailing_list"]

    try:
        mailgun.delete_list(list_address)
    except Exception:
        # Clean up previous test run if interrupted
        pass

    mailgun.create_list(list_address, "Unit test list")
    return list_address


@pytest.fixture()
def domain(mailgun, registry):
    """Outbound domain"""
    secrets = get_secrets(registry)
    return secrets["mailgun.domain"]


@pytest.fixture()
def populated_mailing_list(mailgun, dbsession, registry, mailing_list):
    with transaction.manager:
        create_user(dbsession, registry)

    import_all_users(mailgun, dbsession, mailing_list)
    return mailing_list


@pytest.fixture(scope='session')
def mock_request():
    def mock_requests_response(self, method, url, *args, **kwargs):
        """Mock a response"""
        headers = {'content-type': 'application/json'}
        data = MAILGUN_RESPONSES[url][method]
        body = data['body']
        status_code = data['status_code']
        resp = requests.Response()
        resp.status_code = status_code
        resp.headers = headers
        resp._content = json.dumps(body).encode('utf8')
        return resp
    return mock_requests_response


@pytest.fixture(autouse=True)
def mock_api(monkeypatch, mock_request):
    """Mock all api calls."""
    monkeypatch.setattr(requests.sessions.Session, 'request', mock_request)



# Standard Library
import logging

from .tasks import send_newsletter_task


logger = logging.getLogger(__name__)


def send_newsletter(request, subject: str, preview_email=None, testmode=False, now_=None, import_subscribers=False):
    """Send newsletter.

    The HTML is mangled through premailer.

    Performs the operation asynchronously in a Celery task. The task dispatch is triggered only on commit.

    :param preview_email: Fill in to send a preview
    :param testmode: As in Mailgun parameters
    :param now_: Override timestamp to newsletter state
    :param import_subscribers: Import Websauna userbase as subscribers for the newsletter
    """

    logger.info("Scheduling newsletter task %s, preview %s, import subscribers %s", subject, preview_email, import_subscribers)
    send_newsletter_task.apply_async(args=(subject, preview_email, testmode, now_, import_subscribers), tm=request.tm)



# Standard Library
import sys
from codecs import open
from os import path

from setuptools import find_packages
from setuptools import setup

assert sys.version_info >= (3, 5, 2), "Websauna needs Python 3.5.2 or newer, you have {version}".format(
    version=sys.version_info)

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    README = f.read()

with open(path.join(here, 'CHANGES.rst'), encoding='utf-8') as f:
    CHANGES = f.read()

# trying to run python setup.py install or python setup.py develop
if len(sys.argv) >= 2:
    if sys.argv[0] == "setup.py" and sys.argv[1] in ("install", "develop"):
        # Otherwise so much stuff would be broken later...
        raise RuntimeError(
            "It is not possible to install this package with setup.py. Use pip to install this package as instructed in Websauna tutorial.")

setup(
    name='websauna.newsletter',
    version='1.0a1',
    description='Newsletter add on for Websauna',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Pyramid',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
    ],
    url='https://websauna.org',
    author='Mikko Ohtamaa',
    author_email='mikko@opensourcehacker.com',
    license='MIT',
    keywords='web websauna pyramid',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite='websauna.newsletter',
    install_requires=[
        'websauna[celery]',
        'websauna.magiclogin'
    ],
    extras_require={
        # Dependencies for running test suite
        'test': [
            'codecov',
            'flake8',
            'pytest>=3.0',
            'pytest-runner',
            'coverage',
            'flaky',
            'isort',
            'pytest-cov',
            'pytest-runner',
            'pytest-splinter',
            'pytest-timeout',
            'webtest',
        ],
        # Dependencies to make releases
        'dev': [
            'websauna[dev]'
        ],
    },

    # Define where this application starts as referred by WSGI web servers
    entry_points={
        'paste.app_factory': [
            'main = websauna.newsletter.demo:main'
        ],
    }
)
